@@ -7,9 +7,11 @@ from django.db import models as django_models
 def default_cache_func(
         django_model,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        redis_model,
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
 ):
-    redis_model = get_or_create_redis_model_from_django_model(django_model, redis_root)
+
     redis_dicts = redis_root.get(redis_model, return_dict=True)
     # cache_to_django(
     #     django_model,
@@ -20,6 +22,7 @@ def default_cache_func(
         django_model,
         redis_model,
         redis_root,
+        save_related_models,
         get_or_create_redis_model_from_django_model,
     )
 
@@ -98,7 +101,9 @@ def redis_dict_to_django_params(
                 foreign_key_model = django_field.remote_field.model
                 redis_foreign_key_instance = redis_dict[django_field_name]
                 django_foreign_key_instance = update_or_create_django_instance_from_redis_instance(
-                    redis_foreign_key_instance, foreign_key_model)
+                    redis_foreign_key_instance,
+                    foreign_key_model
+                )
                 django_params[django_field_name] = django_foreign_key_instance
             elif django_field.__class__ == django_models.ManyToManyField:
                 many_to_many_model = django_field.remote_field.model
@@ -106,7 +111,9 @@ def redis_dict_to_django_params(
                 redis_many_to_many_instances = redis_dict[django_field.name]
                 for redis_many_to_many_instance in redis_many_to_many_instances:
                     django_many_to_many_instance = update_or_create_django_instance_from_redis_instance(
-                        redis_many_to_many_instance, many_to_many_model)
+                        redis_many_to_many_instance,
+                        many_to_many_model
+                    )
                     django_many_to_many_instances.append(django_many_to_many_instance)
                 django_many_to_many_params[django_field_name] = django_many_to_many_instances
             else:
@@ -154,12 +161,14 @@ def django_to_cache(
         django_model,
         redis_model,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
 ):
     async def run_django_to_cache(
             django_model,
             redis_model,
             redis_root,
+            save_related_models,
             get_or_create_redis_model_from_django_model,
     ):
         django_instances = await django_sync_to_async_list(django_model.objects.all)
@@ -174,7 +183,8 @@ def django_to_cache(
                     django_instance,
                     redis_model,
                     redis_root,
-                    get_or_create_redis_model_from_django_model
+                    save_related_models,
+                    get_or_create_redis_model_from_django_model,
                 )
             )
             completed += 1
@@ -189,6 +199,7 @@ def django_to_cache(
             django_model,
             redis_model,
             redis_root,
+            save_related_models,
             get_or_create_redis_model_from_django_model,
         )
     )
@@ -198,42 +209,61 @@ async def update_or_create_redis_instance_from_django_instance(
         django_instance,
         redis_model,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        get_or_create_redis_model_from_django_model,
+        save_related_models,
 ):
-    print(django_instance.id)
     redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
     redis_dict = await django_instance_to_redis_params(
         django_instance,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
     )
 
     if not redis_instance:
-        redis_model(
+        fields_to_create = redis_dict
+        redis_instance = redis_model(
             redis_root=redis_root,
             django_id=django_instance.id,
-            **redis_dict
+            **fields_to_create
         ).save()
+        result_redis_instance_id = redis_instance['id']
         print(f'{redis_model} with django_id={django_instance.id} created...')
-        result_redis_instance = redis_root.get(redis_model, django_id=django_instance.id)[0]
     else:
         redis_instance = redis_instance[0]
+        fields_to_update = check_fields_need_to_update(
+            redis_instance,
+            redis_dict,
+        )
         redis_root.update(redis_model, redis_instance,
-                          django_id=django_instance.id,
-                          **redis_dict
+                          **fields_to_update
                           )
-        result_redis_instance = redis_instance
+        result_redis_instance_id = redis_instance['id']
         print(f'{redis_model} with django_id={django_instance.id} existing...')
 
+    return result_redis_instance_id
 
-    return result_redis_instance
+
+def check_fields_need_to_update(
+        redis_instance,
+        redis_dict,
+):
+    fields_to_update = {}
+    for field_name, field_value in redis_dict.items():
+        if field_name not in redis_instance.keys():
+            fields_to_update[field_name] = field_value
+        else:
+            if redis_instance[field_name] != field_value:
+                fields_to_update[field_name] = field_value
+    return fields_to_update
 
 
 async def django_instance_field_to_redis_value(
         django_instance,
         django_field,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
 ):
     django_field_value = await django_sync_to_async_get(django_instance, django_field.name)
     redis_value = django_field_value
@@ -242,22 +272,33 @@ async def django_instance_field_to_redis_value(
             redis_value = None
         else:
             django_foreign_key_instance = django_field_value
-            redis_value = await get_or_create_redis_instance_from_django_instance(
-                django_foreign_key_instance,
-                redis_root,
-                get_or_create_redis_model_from_django_model
-            )
-
+            if save_related_models:
+                redis_foreign_key_instance = await get_or_create_redis_instance_from_django_instance(
+                    django_instance,
+                    redis_root,
+                    save_related_models,
+                    get_or_create_redis_model_from_django_model,
+                )
+                redis_value = redis_foreign_key_instance
+            else:
+                redis_value = django_foreign_key_instance.id
     elif django_field.__class__ == django_models.ManyToManyField:
         django_many_to_many_instances = await django_sync_to_async_list(django_field_value.all)
-        redis_value = [
-            await get_or_create_redis_instance_from_django_instance(
-                django_many_to_many_instance,
-                redis_root,
-                get_or_create_redis_model_from_django_model
-            )
-            for django_many_to_many_instance in django_many_to_many_instances
-        ]
+        if save_related_models:
+            redis_value = [
+                await get_or_create_redis_instance_from_django_instance(
+                    django_instance,
+                    redis_root,
+                    save_related_models,
+                    get_or_create_redis_model_from_django_model,
+                )
+                for django_many_to_many_instance in django_many_to_many_instances
+            ]
+        else:
+            redis_value = [
+                django_many_to_many_instance.id
+                for django_many_to_many_instance in django_many_to_many_instances
+            ]
     elif django_field.__class__.__name__.startswith('Image') or django_field.__class__.__name__.startswith('File'):
         try:
             redis_value = django_field_value.file.path
@@ -269,7 +310,8 @@ async def django_instance_field_to_redis_value(
 async def django_instance_to_redis_params(
         django_instance,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
 ):
     redis_params = {}
     for i, django_field in enumerate(django_instance.__class__._meta.get_fields()):
@@ -278,29 +320,34 @@ async def django_instance_to_redis_params(
                 django_instance,
                 django_field,
                 redis_root,
-                get_or_create_redis_model_from_django_model
+                save_related_models,
+                get_or_create_redis_model_from_django_model,
             )
             redis_params[django_field.name] = redis_param
     return redis_params
 
 
-async def get_or_create_redis_instance_from_django_instance(
+def get_or_create_redis_instance_from_django_instance(
         django_instance,
         redis_root,
-        get_or_create_redis_model_from_django_model
+        save_related_models,
+        get_or_create_redis_model_from_django_model,
 ):
     django_instance_model = django_instance.__class__
     redis_model = get_or_create_redis_model_from_django_model(django_instance_model, redis_root)
     redis_instance = redis_root.get(redis_model, django_id=django_instance.id)
     if not redis_instance:
-        redis_dict = await django_instance_to_redis_params(django_instance, redis_root,
-                                                           get_or_create_redis_model_from_django_model)
-        redis_model(
+        redis_dict = django_instance_to_redis_params(
+            django_instance,
+            redis_root,
+            save_related_models,
+            get_or_create_redis_model_from_django_model,
+        )
+        redis_instance = redis_model(
             redis_root=redis_root,
             django_id=django_instance.id,
             **redis_dict
         ).save()
-        redis_instance = redis_root.get(redis_model, django_id=django_instance.id)[0]
     else:
         redis_instance = redis_instance[0]
 
