@@ -128,10 +128,12 @@ class RedisDateTime(RedisString):
 
 class RedisForeignKey(RedisNumber):
 
-    def __init__(self, model=None, *args, **kwargs):
+    def __init__(self, model=None, save_related_models=False, *args, **kwargs):
         if not issubclass(model, RedisModel):
             raise Exception(f'{model.__name__} class is not RedisModel')
         self.model = model
+        check_types(save_related_models, bool)
+        self.save_related_models = save_related_models
         super().__init__(*args, **kwargs)
 
     def _get_id_from_instance_dict(self):
@@ -170,7 +172,12 @@ class RedisForeignKey(RedisNumber):
         if value != 'null':
             value = super().deserialize_value(value, redis_root)
             check_types(value, int)
-            value = self._get_instance_by_id(value, redis_root)
+            if self.save_related_models:
+                value = self._get_instance_by_id(value, redis_root)
+            else:
+                value = {
+                    'django_id': value
+                }
         return value
 
 
@@ -231,16 +238,17 @@ class RedisList(RedisJson):
 
 class RedisManyToMany(RedisList):
 
-    def __init__(self, model=None, *args, **kwargs):
+    def __init__(self, model=None, save_related_models=False, *args, **kwargs):
         if not issubclass(model, RedisModel):
             raise Exception(f'{model.__name__} class is not RedisModel')
         self.model = model
+        check_types(save_related_models, bool)
+        self.save_related_models = save_related_models
         super().__init__(*args, **kwargs)
 
     def clean(self):
         self.value = self.check_value()
         if self.value not in [None, 'null']:
-            print(self.value)
             self.value = get_ids_from_untyped_data(self.value)
         return super().clean()
 
@@ -261,10 +269,16 @@ class RedisManyToMany(RedisList):
         if value != 'null':
             value = super().deserialize_value(value, redis_root)
             check_types(value, list)
-            values_list = [
-                self._get_instance_by_id(redis_model_instance_dict, redis_root)
-                for redis_model_instance_dict in value
-            ]
+            if self.save_related_models:
+                values_list = [
+                    self._get_instance_by_id(redis_model_instance_dict, redis_root)
+                    for redis_model_instance_dict in value
+                ]
+            else:
+                values_list = [
+                    {'django_id': django_id}
+                    for django_id in value
+                ]
             value = values_list
         return value
 
@@ -440,10 +454,11 @@ class RedisRoot:
         django_models_to_cache = self._get_django_models_to_cache()
         for django_model, cache_conf in django_models_to_cache.items():
             cache_func = cache_conf['cache_func']
+            save_related_models = cache_conf['save_related_models']
             
-            def get_or_create_redis_model_from_django_model(django_model, redis_root):
+            def get_or_create_redis_model_from_django_model(django_model, redis_root, save_related_models):
                 registered_models_names = [model.__name__ for model in redis_root.registered_models]
-                redis_fields = django_fields_to_redis_fields(django_model, redis_root)
+                redis_fields = django_fields_to_redis_fields(django_model, redis_root, save_related_models)
                 if django_model.__name__ in registered_models_names:
                     existing_redis_model_index = registered_models_names.index(django_model.__name__)
                     existing_redis_model = redis_root.registered_models[existing_redis_model_index]
@@ -452,7 +467,7 @@ class RedisRoot:
                     new_redis_model = create_redis_model_from_redis_fields(django_model, redis_fields, redis_root)
                 return new_redis_model
             
-            def django_fields_to_redis_fields(django_model, redis_root):
+            def django_fields_to_redis_fields(django_model, redis_root, save_related_models):
                 redis_fields = {}
                 for django_field in django_model._meta.get_fields():
                     django_field_name = django_field.name
@@ -460,7 +475,7 @@ class RedisRoot:
                         django_field_name = 'django_id'
                         redis_field = RedisNumber(null=False)
                     else:
-                        redis_field = django_field_to_redis_field(django_field, redis_root)
+                        redis_field = django_field_to_redis_field(django_field, redis_root, save_related_models)
                     if redis_field:
                         redis_fields[django_field_name] = redis_field
                 return redis_fields
@@ -470,7 +485,7 @@ class RedisRoot:
                 redis_root.register_models([new_redis_model])
                 return new_redis_model
 
-            def django_field_to_redis_field(django_field, redis_root):
+            def django_field_to_redis_field(django_field, redis_root, save_related_models):
 
                 field_mapping = {
                     django_models.BooleanField: RedisBool,
@@ -509,11 +524,11 @@ class RedisRoot:
                         {}
                     )
 
-                    redis_field_params = django_field_params_to_redis_field_params(django_field, redis_root)
+                    redis_field_params = django_field_params_to_redis_field_params(django_field, redis_root, save_related_models)
                     redis_field = redis_field_class(**redis_field_params)
                 return redis_field
 
-            def django_field_params_to_redis_field_params(django_field, redis_root):
+            def django_field_params_to_redis_field_params(django_field, redis_root, save_related_models):
                 allowed_params = ['null', 'default', 'choices', 'remote_field']
                 allowed_redis_params = {}
                 for django_field_param_name in django_field.__dict__.keys():
@@ -539,6 +554,7 @@ class RedisRoot:
                                     value = get_or_create_redis_model_from_django_model(value, redis_root)
                                     if value is not None:
                                         allowed_redis_params[redis_field_param_name] = value
+                                        allowed_redis_params['save_related_models'] = save_related_models
                         else:
                             allowed_redis_params[redis_field_param_name] = value
                 return allowed_redis_params
@@ -554,10 +570,14 @@ class RedisRoot:
                 redis_root.registered_models = new_registered_models
                 return new_redis_model
 
+            redis_model = get_or_create_redis_model_from_django_model(django_model, self, save_related_models)
+
             cache_func(
                 django_model,
                 self,
-                get_or_create_redis_model_from_django_model
+                redis_model,
+                save_related_models,
+                get_or_create_redis_model_from_django_model,
             )
             print(f'Cached {django_model}')
 
@@ -597,8 +617,8 @@ class RedisRoot:
         cache_conf = {
             'enabled': False,
             'ttl': 60 * 5,
+            'save_related_models': True,
             'cache_func': default_cache_func,
-            'prefix': f'{self.prefix}-{name}-cache',
         }
         if user_cache_conf is not None:
             if type(user_cache_conf) == dict:
@@ -612,23 +632,17 @@ class RedisRoot:
                         cache_conf['ttl'] = user_cache_conf['ttl']
                     else:
                         raise Exception(f'{name} -> cache config -> ttl must be int')
-                if 'django_model' in user_cache_conf.keys():
-                    if issubclass(user_cache_conf['django_model'], django_models.Model):
-                        cache_conf['django_model'] = user_cache_conf['django_model']
+                if 'save_related_models' in user_cache_conf.keys():
+                    if type(user_cache_conf['save_related_models']) == bool:
+                        cache_conf['save_related_models'] = user_cache_conf['save_related_models']
                     else:
-                        raise Exception(
-                            f'{name} -> cache config -> django_model must be subclass of django.db.models Model')
+                        raise Exception(f'{name} -> cache config -> save_related_models must be int')
                 if 'cache_func' in user_cache_conf.keys():
                     if type(user_cache_conf['cache_func']) == Callable:
                         cache_conf['cache_func'] = user_cache_conf['cache_func']
                     else:
                         raise Exception(
                             f'{name} -> cache config -> cache_func must be callable function')
-                if 'prefix' in user_cache_conf.keys():
-                    if type(user_cache_conf['prefix']) == str:
-                        cache_conf['prefix'] = user_cache_conf['prefix']
-                    else:
-                        raise Exception(f'{name} -> cache config -> prefix must be str')
             else:
                 raise Exception(f'{name} -> cache config must be dict')
 
@@ -884,8 +898,8 @@ class RedisRoot:
                 if updated_data is not None:
                     updated_datas.append(updated_data)
         else:
-            ids_to_delete = get_ids_from_untyped_data(instances)
-            for instance_id in ids_to_delete:
+            ids_to_update = get_ids_from_untyped_data(instances)
+            for instance_id in ids_to_update:
                 for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}:*'):
                     updated_data = self._update_by_key(key, fields_to_update, renew_ttl, new_ttl)
                     if updated_data is not None:
@@ -916,6 +930,27 @@ class RedisRoot:
                 for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}:*'):
                     self._delete_by_key(key)
 
+    # def get_id_from_django_id(self, model, django_id):
+    #     model_name = model.__name__
+    #     check_types(django_id, int)
+    #     result = False
+    #     for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:*:django_id'):
+    #         prefix, model_name, instance_id, field_name = key.split(':')
+    #         value = int(self.redis_instance.get(key))
+    #         if value == django_id:
+    #             result = instance_id
+    #     return result
+    #
+    # def is_instance_edited_by_id(self, model, instance_id):
+    #     model_name = model.__name__
+    #     check_types(instance_id, int)
+    #     result = False
+    #     for key in self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}:edited'):
+    #         prefix, model_name, instance_id, field_name = key.split(':')
+    #         value = bool(int(self.redis_instance.get(key)))
+    #         result = value
+    #     return result
+
 
 class RedisModel:
     id = RedisId()
